Package: crisprBowtie
Version: 1.15.0
Date: 2025-07-22
Title: Bowtie-based alignment of CRISPR gRNA spacer sequences
Authors@R: c(
    person("Jean-Philippe", "Fortin", email = "fortin946@gmail.com", role = c("aut", "cre")))
Depends: methods
Imports: BiocGenerics, Biostrings, BSgenome, crisprBase (>= 0.99.15),
        Seqinfo, GenomicRanges, IRanges, Rbowtie, readr, stats,
        stringr, utils
Suggests: BiocStyle, BSgenome.Hsapiens.UCSC.hg38, knitr, rmarkdown,
        testthat
biocViews: CRISPR, FunctionalGenomics, Alignment
Description: Provides a user-friendly interface to map on-targets and off-targets
    of CRISPR gRNA spacer sequences using bowtie. The alignment is fast, 
    and can be performed using either commonly-used or custom CRISPR nucleases.
    The alignment can work with any reference or custom genomes.
    Both DNA- and RNA-targeting nucleases are supported.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.2.1
VignetteBuilder: knitr
BugReports: https://github.com/crisprVerse/crisprBowtie/issues
URL: https://github.com/crisprVerse/crisprBowtie
git_url: https://git.bioconductor.org/packages/crisprBowtie
git_branch: devel
git_last_commit: 5838b33
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-04 21:58:49 UTC; biocbuild
Author: Jean-Philippe Fortin [aut, cre]
Maintainer: Jean-Philippe Fortin <fortin946@gmail.com>
