# Generated by roxygen2: do not edit by hand

export(runBowtie)
export(runCrisprBowtie)
importFrom(BSgenome,getSeq)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(Biostrings,DNAString)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,replaceLetterAt)
importFrom(Biostrings,reverseComplement)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(IRanges,promoters)
importFrom(Rbowtie,bowtie)
importFrom(Seqinfo,seqlengths)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(crisprBase,"spacerLength<-")
importFrom(crisprBase,extractPamFromTarget)
importFrom(crisprBase,extractProtospacerFromTarget)
importFrom(crisprBase,getTargetRanges)
importFrom(crisprBase,hasSpacerGap)
importFrom(crisprBase,isRnase)
importFrom(crisprBase,nucleaseName)
importFrom(crisprBase,pamIndices)
importFrom(crisprBase,pamLength)
importFrom(crisprBase,pamSide)
importFrom(crisprBase,pams)
importFrom(crisprBase,spacerLength)
importFrom(methods,is)
importFrom(readr,read_delim)
importFrom(stats,complete.cases)
importFrom(stringr,str_count)
importFrom(stringr,str_extract)
importFrom(utils,data)
importFrom(utils,write.table)
